/*
 evalcorr

   Authors:  Duncan John Fyfe - LUX
         
   Description: Evaluate the output of eposcorr
 
   Comments: 
 
  $Id: module.cc,v 1.5 2000/10/02 08:30:13 gvacanti Exp $
*/

#include <string>

#include "param.h"
#include "errstr.h"
#include "Dal.h"
#include "stringutil.h"

#include "Evalcorr.h"

#include <Tracer.h>
#include <errstr.h>

#include "evalcorr.h"

typedef map<string,double> Kwlist;

void evalcorr ()
{
PROC

	// Initialize

	string syserrcc_name = "SYSERRCC";
	double syserrcc_value = 0.0;
	string syserrcc_comment = "[arcsec] systematic srce posn error for cat corrln";

	string poscorok_name = "POSCOROK";
	bool poscorok_value = 0;
	string poscorok_comment = "eposcorr positions OK?";
		
	// Command line parameters
	string input = stringParameter("input");
	bool amendinput = booleanParameter("amendinput");

	bool withoutput = booleanParameter("withoutput");
	string output = "";
	if (withoutput) {
			
		output = stringParameter("output");
	}

	bool forcesyserrcc =  booleanParameter("forcesyserrcc");
	double syserrcc = 0.0;
	if (forcesyserrcc) syserrcc = realParameter("syserrcc");

	bool forceposcorok =  booleanParameter("forceposcorok");
	bool poscorok = 0;
	if (forceposcorok) poscorok = booleanParameter("poscorok");

	bool forcecopycols =  booleanParameter("forcecopycols");

	double corrsyserrcc = realParameter("corrsyserrcc");
	double uncorrsyserrcc = realParameter("uncorrsyserrcc");
	double radiusthreshold = realParameter("radiusthreshold");
	double likhoodthreshold = realParameter("likhoodthreshold");
	
	if (input == output) {
		errstr << "Values of parameters input and output are identical" << Sas::error("InputIdenticaltoOutput");
		return;
	}
	//
	//Open source list
	
	DataSet *set;

	bool makeamends=0 ;
	if (withoutput) {

		// output source list provided, copy input dataset and open in Modify mode

		errstr<<"Cloning dataset ... "<<Sas::message(Msg::App, Msg::Sparse);
	  	set = dataSetServer->clone(input, output, Dal::Modify);
		errstr<<"done."<<Sas::message(Msg::App,Msg::Sparse);
		makeamends=1;

	} else {

		if (amendinput) {
		
			// want to make changes directly to input source list
			// open in Modify mode

			errstr << "Opening " << input << " in Modify mode" << Sas::message(Msg::App, Msg::Sparse);
			set = dataSetServer->open(input, Dal::Modify );
			makeamends=1;
		}
		else {
			
			// do not want to change input source list.  
			// (actions are written out but not applied)
			
			errstr << "Opening " << input << " NOT in Modify mode" << Sas::message(Msg::App, Msg::Sparse);
    		set = dataSetServer->open(input, Dal::Read);
			makeamends=0;
		}
	}
	
	// Read Keywords	
	
	Table	*table;
	table = set->table("SRCLIST");

	Kwlist read_kw;
	ReadKeywords(table,read_kw);

	// Evaluate correlation unless all values are forced
	
	if (forcesyserrcc && forceposcorok ) {
			
		syserrcc_value = syserrcc;
		poscorok_value = poscorok;
	}
	else {

		Evaluate(read_kw , syserrcc_value , poscorok_value , corrsyserrcc , uncorrsyserrcc , radiusthreshold , likhoodthreshold);
		if (forcesyserrcc) syserrcc_value = syserrcc;
		if (forceposcorok) poscorok_value = poscorok;
	}

	// Copy columns
	// We replace ra_corr with ra etc when (poscorr=0 && !forceposcorok) || forcecopycols = 1
	// ie. do not copycols if poscorok forced unless asked to.

	if ((!poscorok_value && !forceposcorok) || forcecopycols) {

		CopyCols(table,makeamends);
	}
	
	// write new keywords
	// SYSERRCC
	if(makeamends) {	
				
		(table->hasAttribute(syserrcc_name))
			? *table->attribute(syserrcc_name) = syserrcc_value 
			: *table->addAttribute(syserrcc_name , syserrcc_comment) = syserrcc_value;
		;
	}
	errstr << "wrote attribute " << syserrcc_name << " = " << toString(syserrcc_value) << Sas::message(Msg::App, Msg::Sparse);
	
	// POSCOROK
	if(makeamends) {	
				
		(table->hasAttribute(poscorok_name))
			? *table->attribute(poscorok_name) = poscorok_value 
			: *table->addAttribute(poscorok_name,    poscorok_comment) = poscorok_value;
		;
	}
	errstr << "wrote attribute " << poscorok_name << " = " << toString(poscorok_value) << Sas::message(Msg::App, Msg::Sparse);
	
	dataSetServer->close(set);

ENDPROC
} // evalcorr()
